local *

{
  :BuffFrame
  :CastingBarFrame
  :MuteSoundFile
  :TemporaryEnchantFrame
  :UIErrorsFrame
  :UnmuteSoundFile
  :idTweaks
  :idTweaksModule
  :idTweaksOptions
} = _G
{
  :Option
  :ToggleOption
} = idTweaksOptions

class BlizzardUI extends idTweaksModule
  new: (addon) =>
    -- print "BlizzardUI#new"

    super addon, "BlizzardUI", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0"

  set_options: =>
    -- print "BlizzardUI#set_options"

    super!

    @options\add_option Option @, "buff_frame", "Buffs", "Customize the Buff frame", {},
      hide: ToggleOption @, "hide", "Hide", "Hide the frame"

    @options\add_option Option @, "casting_bar_frame", "CastingBar", "Customize the Casring Bar Frame frame", {},
      hide: ToggleOption @, "hide", "Hide", "Hide the frame"

    @options\add_option Option @, "errors_frame", "Error", "Customize the center screen error messages", {},
      out_of_range: ToggleOption @, "out_of_range", "Out of range", "Out of range"
      spell_is_not_ready_yet: ToggleOption @, "spell_is_not_ready_yet", "Spell is not ready yet", "Spell is not ready yet"
      target_needs_to_be_in_front_of_you: ToggleOption @, "target_needs_to_be_in_front_of_you", "Target needs to be in font of you", "Target needs to be in front of you"
      target_too_close: ToggleOption @, "target_too_close", "Target too close", "Target too close"
      there_is_nothing_to_attack: ToggleOption @, "there_is_nothing_to_attack", "There is nothing to attack", "There is nothing to attack"
      you_are_facing_the_wrong_way: ToggleOption @, "you_are_facing_the_wrong_way", "You are facing the wrong way", "You are facing the wrong way"
      you_are_too_far_away: ToggleOption @, "you_are_too_far_away", "You are too far away", "You are too far away"
      you_cant_do_that_yet: ToggleOption @, "you_cant_do_that_yet", "You cant do that yet", "You cant do that yet"
      you_have_no_target: ToggleOption @, "you_have_no_target", "You have no target", "You have no target"

    @options\add_option Option @, "sounds", "Sounds", "Sounds", {},
      fizzle: ToggleOption @, "fizzle", "Fizzle (disabled)", "Played when a spell fails"

  check: =>
    -- print "BlizzardUI#check"

    @buff_frame_check!
    @casting_bar_frame_check!
    @sounds_check!

  enable: =>
    -- print "BlizzardUI#enable"

    @buff_frame_hooked = false
    @casting_bar_frame_hooked = false

    add_message = (frame, message, red, green, blue, message_id, hold_time) ->
      found = false
      for k, v in pairs @db.errors_frame
        option = @options.args.errors_frame.args[k]
        found = true if v and option and message == option.name
      @ace_module.hooks[frame].AddMessage frame, message, red, green, blue, message_id, hold_time if not found
    @ace_module\RawHook UIErrorsFrame, "AddMessage", add_message, true

    @check!

  disable: =>
    -- print "BlizzardUI#disable"

    @buff_frame_disable!
    @casting_bar_frame_disable!

    @ace_module\Unhook UIErrorsFrame, "AddMessage"

  buff_frame_check: =>
    -- print "BlizzardUI#buff_frame_check"

    if @db.buff_frame.hide
      @buff_frame_enable!
    else
      @buff_frame_disable!

  buff_frame_enable: =>
    -- print "BlizzardUI#buff_frame_enable"

    unless @buff_frame_hooked
      @ace_module\SecureHook BuffFrame, "Show", (frame) ->
        BuffFrame\Hide!
      @ace_module\SecureHook TemporaryEnchantFrame, "Show", (frame) ->
        TemporaryEnchantFrame\Hide!
      @buff_frame_hooked = true

      BuffFrame\Hide!
      TemporaryEnchantFrame\Hide!

  buff_frame_disable: =>
    -- print "BlizzardUI#buff_frame_disable"

    @ace_module\Unhook BuffFrame, "Show"
    @ace_module\Unhook TemporaryEnchantFrame, "Show"
    @buff_frame_hooked = false

    BuffFrame\Show!
    TemporaryEnchantFrame\Show!

  casting_bar_frame_check: =>
    -- print "BlizzardUI#casting_bar_frame_check"

    if @db.casting_bar_frame.hide
      @casting_bar_frame_enable!
    else
      @casting_bar_frame_disable!

  casting_bar_frame_enable: =>
    -- print "BlizzardUI#casting_bar_frame_enable"

    unless @casting_bar_frame_hooked
      @ace_module\SecureHook CastingBarFrame, "Show", (frame) ->
        CastingBarFrame\Hide!
      @casting_bar_frame_hooked = true

      CastingBarFrame\Hide!

  casting_bar_frame_disable: =>
    -- print "BlizzardUI#casting_bar_frame_disable"

    @ace_module\Unhook CastingBarFrame, "Show"
    @casting_bar_frame_hooked = false

  sounds_check: =>
    -- print "BlizzardUI#sounds_check"

    if @db.sounds.fizzle
      MuteSoundFile(569772)
      MuteSoundFile(569773)
      MuteSoundFile(569774)
      MuteSoundFile(569775)
      MuteSoundFile(569776)
    else
      UnmuteSoundFile(569772)
      UnmuteSoundFile(569773)
      UnmuteSoundFile(569774)
      UnmuteSoundFile(569775)
      UnmuteSoundFile(569776)

idTweaks\add_module BlizzardUI idTweaks
